﻿local F = far.Flags

local C = far.Colors
local Color = far.AdvControl(F.ACTL_GETCOLOR,C.COL_HELPTOPIC)
local function Hint(str,state)
  editor.Redraw()
  state = state and (state==0 and ":off" or ":on") or " "
  far.Text(1,0,Color,":"..str..state)
  far.Text()
  win.Sleep(400)
end

---------------------------------------------
local one_off_persistance = true
---------------------------------------------
local PersistentBlocks = 2 --Постоянные блоки
local getValue = -1 --Значение Value может быть равно -1, в этом случае функция ничего не изменит, а вернёт текущее значение настройки.
local function isPersistentGlobal()
  return Far.Cfg_Get("Editor","PersistentBlocks")=="true"
end

local keep_persistent
local function persistent(on) --Persistent blocks
  if isPersistentGlobal() then return true end
  if on==false and keep_persistent then
    keep_persistent = false
    on = nil
  else
    keep_persistent = on
  end
  return Editor.Set(PersistentBlocks,on==nil and getValue or on and 1 or 0)==1
end

local E = {}
for i=1,far.AdvControl(F.ACTL_GETWINDOWCOUNT) do
  local w = far.AdvControl(F.ACTL_GETWINDOWINFO,i)
  if w.Type==F.WTYPE_EDITOR then E[w.Id] = {} end
end

local function neq(sel1,sel2)
  if not (sel1 and sel2) then return sel1==sel2 and 0 or 5 end
  local diff = 0
  for k,v in pairs(sel1) do
    if sel2[k]~=v then diff = diff+1 end
  end
  return diff
end
Event { description="Ctrl-K-x internal handler";
  group="EditorEvent";
  uid="7FA6E14F-A4E7-40AD-896A-BB78839EA5BE";
  action=function(id,Event,param)
    if Event==F.EE_READ then
      E[id] = {}
    elseif Event==F.EE_CLOSE then
      E[id] = nil
    elseif Event==F.EE_REDRAW and E[id] then
      local sel = editor.GetSelection()
      local diff = neq(E[id].LastSel,sel)
      if diff>0 then
        if diff>1 and one_off_persistance then
          persistent(false)
        end
        --todo save to Selection ??if was persistant
        if not sel then
          E[id].PrevSel = E[id].LastSel
        end
        E[id].LastSel = sel
      end
    end
  end;
}

local function getSel() --side-effect: saves current selection as new block (affects: begin,end,toggle,changetype)
                        --todo save to Selection ??if was persistant
  local info = editor.GetInfo()
  local id = info.EditorID
  if info.BlockType~=F.BTYPE_NONE then
    E[id].Selection = editor.GetSelection()
  end
  E[id].Selection = E[id].Selection or {
    BlockType=F.BTYPE_NONE,
    StartLine=1,
    StartPos=1,
    EndLine=1,
    EndPos=0,
  }
  return info.CurLine,info.CurPos,E[id].Selection,info.BlockType
end

local btype = F.BTYPE_STREAM
local function changeBlock(sel)
  local startpos = btype==F.BTYPE_STREAM and sel.StartPos
                or btype==F.BTYPE_COLUMN and editor.RealToTab(nil,sel.StartLine,sel.StartPos)
  local endpos   = btype==F.BTYPE_STREAM and sel.EndPos
                or btype==F.BTYPE_COLUMN and editor.RealToTab(nil,sel.EndLine,sel.EndPos+1)-1 --http://forum.farmanager.com/viewtopic.php?p=121441#p121441
  local sel2 = {
    BlockType=btype,
    BlockStartPos=startpos,
    BlockWidth=endpos-startpos+1,
    BlockStartLine=sel.StartLine,
    BlockHeight=sel.EndLine-sel.StartLine+1,
  }
  local visible = sel2.BlockWidth~=0 or sel2.BlockHeight>1
  visible = editor.Select(nil,sel2) and visible
  if not visible then editor.Select(nil,F.BTYPE_NONE) end
  return visible
end

local function markLine()
  editor.Select(nil,{
    BlockType=F.BTYPE_STREAM,
    BlockStartLine=0,
    BlockStartPos=1,
    BlockWidth=-1,
    BlockHeight=1,
  })
end;

Macro { description="^KB  Mark block begin";
  area="Editor"; key="CtrlK-B";
  uid="6DC7442D-D7D0-4C5F-9C82-FE272BC4012D";
  action=function()
    local line,pos,sel = getSel()
    sel.StartLine = line
    sel.StartPos = pos
    persistent(changeBlock(sel))
    Hint"block:begin"
  end;
}

Macro { description="^KK  Mark block end ";
  area="Editor"; key="CtrlK-K";
  uid="7942DD2A-EE17-49A1-87A8-05D2CB62102B";
  action=function()
    local line,pos,sel = getSel()
    sel.EndLine = line
    sel.EndPos = pos-1
    persistent(changeBlock(sel))
    Hint"block:end"
  end;
}

Macro { description="^KC  Copy block";
  area="Editor"; key="CtrlK-C";
  uid="278BB5B6-9387-4730-BF0D-0ECE15EF9A10";
  action=function() --copyBlockSmart
    local prev = persistent(true)
    if not Object.Selected then
      editor.SetPosition(nil,{CurPos=1}); markLine()
      Keys"CtrlP"
      editor.Select(nil,F.BTYPE_NONE)
    else
      Keys"CtrlP"
    end
    persistent(prev)
  end;
}

Macro { description="^KV  Move block";
  area="Editor"; key="CtrlK-V";
  uid="6D9F1D94-73D3-480E-B3A0-D3F167FC2154";
  action=function() persistent(persistent(),Keys"CtrlM") end;
}

Macro { description="^KT  Mark single word";
  area="Editor"; key="CtrlK-T";
  uid="39D99815-D954-40A6-BA54-B620C1776FAD";
  action=function() Keys"SelWord"; persistent(true) end;
}

--todo ?? LeftPos
local function isVisible(sel) --checks block visibility on current screen
  local info = editor.GetInfo()
  local top = info.TopScreenLine
  local bottom = top+info.WindowSizeY-1
  if sel.StartLine>=top and sel.StartLine<=bottom or --todo check
     sel.EndLine>=top and sel.EndLine<=bottom or
     sel.StartLine<top and sel.EndLine>bottom then
    return true
  end
end
Macro { description="^KY  Delete block";
  area="Editor"; key="CtrlK-Y";
  uid="36D11712-9363-4399-B316-745F4C37EF55";
  action=function() --delBlockSmart
    local sel = editor.GetSelection()
    if sel then
      if isVisible(sel) then -- remove block only if it visible on current screen
        if editor.DeleteBlock() then
          persistent(false)
        end
      else                   -- (othervise goto block position) 
        local TopScreenLine = sel.StartLine-3
        editor.SetPosition (nil,{
          CurPos=sel.EndPos+1,
          CurLine=sel.EndLine,
          TopScreenLine=TopScreenLine>0 and TopScreenLine or 1
        })
      end
    end
  end;
}

--^KR Read block from disk
--^KW Write block to disk
--^KP Print block

--todo ??http://forum.farmanager.com/viewtopic.php?f=60&t=7703
Macro { description="^KI  Indent block";
  area="Editor"; key="CtrlK-I";
  uid="B1B58440-FABA-4267-86EA-696C50EB9977";
  action=function()
    if editor.GetInfo().BlockType~=F.BTYPE_NONE then Keys"AltI" end
  end;
}

Macro { description="^KU  Unindent block";
  area="Editor"; key="CtrlK-U";
  uid="032BFA41-F0A5-4879-B14D-8AADBDBE7018";
  action=function()
    if editor.GetInfo().BlockType~=F.BTYPE_NONE then Keys"AltU" end
  end;
}

Macro { description="^KH  Hide/display block";
  area="Editor"; key="CtrlK-H";
  uid="BC65E5C2-A188-4D32-8C22-7EF9F8C7BC14";
  action=function() --toggleBlock
    local sel,BlockType = select(3,getSel())
    local visible = false
    if BlockType==F.BTYPE_NONE then
      visible = changeBlock(sel)
      BlockType = not visible and 0 or 1
    else
      editor.Select(nil,F.BTYPE_NONE)
      BlockType = 0
    end
    persistent(visible)
    Hint("block",BlockType)
  end;
}

local AKey --http://bugs.farmanager.com/view.php?id=2309
Macro { description="^K-n Set place marker";
  area="Editor"; key="CtrlK-1 CtrlK-2 CtrlK-3 CtrlK-4 CtrlK-5 CtrlK-6 CtrlK-7 CtrlK-8 CtrlK-9 CtrlK-0";
  uid="D55309A1-4901-4799-B4A3-8C4149B37E01";
  condition=function(key) AKey = key; return true end;
  action=function() Keys(AKey:gsub("K-","Shift")) end;
}

Macro { description="^KN  Turns column mode on/off";
  area="Editor"; key="CtrlK-N";
  uid="B2877D98-7FBF-4112-9C56-F4DB65DE88B4";
  action=function() --toggleBtype
    local sel,BlockType = select(3,getSel())
    if BlockType~=F.BTYPE_NONE then
      btype = BlockType==F.BTYPE_STREAM and F.BTYPE_COLUMN or F.BTYPE_STREAM
      persistent(changeBlock(sel))
    else
      btype = btype==F.BTYPE_STREAM and F.BTYPE_COLUMN or F.BTYPE_STREAM
    end
    Hint("block: column",btype==F.BTYPE_COLUMN and 1 or 0)
  end;
}

Macro { description="^KL  Mark line";
  area="Editor"; key="CtrlK-L";
  uid="B6AC0E34-D7ED-41F2-AC6E-F6CAA15F934F";
  action=function() markLine(); persistent(true) end;
}

--todo ??http://plugring.farmanager.com/plugin.php?pid=267&l=ru
local function changeCase(cmd)
  Plugin.Call("0E92FC81-4888-4297-A85D-31C79E0E0CEE",cmd) --Change Case
end
Macro { description="^KE  lower [Change Case plugin]";
  area="Editor"; key="CtrlK-E";
  uid="C847261E-7299-4AF1-A3BF-8E2386B081D3";
  action=function() changeCase"Lower" end;
}

Macro { description="^KF  UPPER [Change Case plugin]";
  area="Editor"; key="CtrlK-F";
  uid="304A61E4-7372-4A3A-B15B-A1F22F23916F";
  action=function() changeCase"Upper" end;
}

Macro { description="^KX  Overwrite block with clipboard content";
  area="Editor"; key="CtrlK-X";
  uid="335A521A-4226-4AAC-9A27-2AD93A59EB37";
  action=function() persistent(persistent(),Keys"CtrlD ShiftIns") end;
}

Macro { description="^KP  Mark last block";
  area="Editor"; key="CtrlK-P";
  uid="4B8198E4-3D3F-4B88-AFF6-F34427D449DE";
  action=function()
    local info = editor.GetInfo()
    local PrevSel = E[info.EditorID].PrevSel
    if PrevSel then
      if neq(PrevSel,editor.GetSelection())>0 then
        persistent(changeBlock(PrevSel)) --!!http://bugs.farmanager.com/view.php?id=2756
        Hint("last block restored")
      else
        editor.Select(nil,F.BTYPE_NONE)
      end
    elseif info.BlockType~=F.BTYPE_NONE then
      editor.Select(nil,F.BTYPE_NONE)
    else
      Hint("no last block")
      mf.beep()
    end
  end;
}

-----------------------------------------------------------------------
NoMacro { description="Mark last block";
  area="Editor"; key="CtrlU"; --flags="NoEVSelection";
  uid="65BB4607-14A9-4E0F-8437-5FBE3F0EB3F2";
  action=markLastBlock;
}
