local F = far.Flags

local C = far.Colors
local Color = far.AdvControl(F.ACTL_GETCOLOR,C.COL_HELPTOPIC)
local function Hint(str,state)
  editor.Redraw()
  state = state and (state==0 and ":off" or ":on") or " "
  far.Text(1,0,Color,":"..str..state)
  far.Text()
  win.Sleep(400)
end

-------------------------------------------------------------
--Editor.Set constants: http://api.farmanager.com/ru/macro/macrocmd/prop_func/editor.html#editor.set
--EditorSetParameter structure: http://api.farmanager.com/ru/structures/editorsetparameter.html
local ExpandTabs = 1 --  :
local PersistentBlocks = 2 -- 
--local DelRemovesBlocks = 3 -- Delete  .    , Delete    ,     .
local AutoIndent = 4 --.      .
local ShowWhiteSpace = 20 --    .
local CursorBeyondEOL = 7 --     
--local SearchPickUpWord = 19 --C  .    /      ,    .
--local SearchSelFound = 17 --  -    .
--SearchCursorAtEnd
--local ShowScrollBar = 15 --    
--local SearchRegexp = 18 --     

--local BSLikeDel = 8 --  BackSpace (hlf:@Editor.BSLikeDel).
local CharCodeBase = 9 --        (hlf:@Editor.CharCodeBase)
--local SaveEditorPos = 10   
--local SaveEditorShortPos = 11 -- 
--local AllowEmptySpaceAfterEof = 14  -- hlf:@Editor.AllowEmptySpaceAfterEof
--local WordDiv = 12 --   (hlf:@Editor.WordDiv) --  : ~!%^&*()+|{}:"<>?`-=\[];',./
--ShowKeyBar - CtrlB
--ShowTitleBar - CtrlShiftB

local getValue = -1 -- Value    -1,       ,     .

Macro { description="^OO  Current editor options";
  area="Editor"; key="CtrlO-O";
  uid="A8393E07-B4E4-4B7D-814D-DC1895E3E371";
  action=function() Keys"AltShiftF9" end;
}

local KeyRec = far.NameToInputRecord ("F3")
local Key = {VirtualKeyCode=KeyRec.VirtualKeyCode;
            ControlKeyState=KeyRec.ControlKeyState;
}
local function setKeyBar(persistent) --todo Mantis#0001620
  Key.Text = persistent and "Pers.-" or "Pers.+"
  Key.LongText = persistent and "Persistent off" or "Persistent on"
  editor.SetKeyBar(nil,{Key})
end
Macro { description="^OP  Persistent blocks on/off";
  area="Editor"; key="CtrlO-P F3 CtrlAltP";
  uid="C68F1293-B5CB-4701-8811-45A606573615";
  action=function()
    local NewState = 1-Editor.Set(PersistentBlocks,getValue)
    Editor.Set(PersistentBlocks,NewState)
    setKeyBar(NewState==1)
    Hint("Persistent blocks",NewState)
    --todo ?? save Selection now
  end;
}

Event { description="show persistent blocks state on keybar";
  group="EditorEvent";
  uid="08F48615-5C25-4A49-8FC4-5F0030186D37";
  action=function(id,Event,param)
    if Event==F.EE_READ then
      setKeyBar(Editor.Set(PersistentBlocks,getValue)==1)
    end
  end;
}

Macro { description="^OI  Autoindent on/off";
  area="Editor"; key="CtrlO-I";
  uid="6FA49B4C-7D5C-4241-BCB5-C162B6596DAD";
  action=function()
    --F.ESPT_AUTOINDENT
    NewState = 1-Editor.Set(AutoIndent,getValue)
    Editor.Set(AutoIndent,NewState)
    Hint("AutoIndent",NewState)
  end;
}

Macro { description="^OT  Tab mode on/off";
  area="Editor"; key="CtrlO-T";
  uid="61F1D17F-877F-41DB-AAD2-685CA18F25BD";
  action=function()
    --F.ESPT_EXPANDTABS
    local NewState = Editor.Set(ExpandTabs,getValue)==0 and 2 or 0
    Editor.Set(ExpandTabs,NewState)
    Hint("ExpandTabs",NewState)
  end;
}

Macro { description="^OS  Show white space";
  area="Editor"; key="CtrlO-S";
  uid="CD4385F4-4AAF-4F2A-A0CE-C7BDAB51E599";
  action0=function() --cyclic
    local state = Editor.Set(ShowWhiteSpace,getValue)+1
    Editor.Set(ShowWhiteSpace,state>2 and 0 or state)
  end;
  action=function()
    --F.ESPT_SHOWWHITESPACE
    local NewState = Editor.Set(ShowWhiteSpace,getValue)==0 and 2 or 0
    Editor.Set(ShowWhiteSpace,NewState)
  end;
}

Macro { description="^OC  CharCodeBase";
  area="Editor"; key="CtrlO-C";
  uid="45AA206B-8390-4B1D-A628-97599AD9E68F";
  action=function()
    --F.ESPT_CHARCODEBASE
    local state = Editor.Set(CharCodeBase,getValue)
    Editor.Set(CharCodeBase,state==2 and 1 or 2)
  end;
}

Macro { description="^OB  Cursor beyond end of line on/off";
  area="Editor"; key="CtrlO-B";
  uid="1B9150E2-2F91-43D1-B806-0BABEAC42B12";
  action=function()
    --F.ESPT_CURSORBEYONDEOL
    local NewState = 1-Editor.Set(CursorBeyondEOL,getValue)
    Editor.Set(CursorBeyondEOL,NewState)
    Hint("CursorBeyondEOL",NewState)
  end;
}

Macro { description="^OW  Auto wrap toggle [Auto wrap plugin]";
  area="Editor"; key="CtrlO-W";
  uid="AA64946A-397C-4552-94E6-2A7F29A4D800";
  action=function()
    if Plugin.Call("DEEC52C3-AC76-4AD3-A6EF-CAFC33BD4C05") then --Auto wrap
      Keys"Space Enter"
    end
  end;
}

local function markLine()
  editor.Select(nil,{
    BlockType=F.BTYPE_STREAM,
    BlockStartLine=0,
    BlockStartPos=1,
    BlockWidth=-1,
    BlockHeight=1,
  })
end;

Macro { description="^OJ  Justify Paragraph [Align block plugin]";
  area="Editor"; key="CtrlO-J";
  uid="EBAD6F31-40E0-4A90-A59B-EF8F2D421B3E";
  action=function()
    if editor.GetInfo().BlockType==F.BTYPE_NONE then markLine() end
    editor.UndoRedo(nil,F.EUR_BEGIN)
    Plugin.SyncCall("B076F0B0-90AE-408C-AD09-491606F09435") --Align block
    editor.UndoRedo(nil,F.EUR_END)
  end
}