local F = far.Flags

local C = far.Colors
local Color = far.AdvControl(F.ACTL_GETCOLOR,C.COL_HELPTOPIC)
local function Hint(str,state)
  editor.Redraw()
  state = state and (state==0 and ":off" or ":on") or " "
  far.Text(1,0,Color,":"..str..state)
  far.Text()
  win.Sleep(400)
end

Macro { description="^QS  Go to start of Line";
  area="Editor"; key="CtrlQ-S";
  uid="195B96FE-779F-4A6C-943D-E31F6EC5E4E4";
  action=function() Keys"Home" end;
}

Macro { description="^QD  Go to end of line";
  area="Editor"; key="CtrlQ-D";
  uid="1C9FEFD1-47D8-4BBD-BC36-C54553D07CB1";
  action=function() Keys"End" end;
}

Macro { description="^QE  Go to top of window";
  area="Editor"; key="CtrlQ-E";
  uid="F64AA6E5-165C-42DF-8F72-87953FEE274F";
  action=function() Keys"CtrlN" end; --FAR bug: http://bugs.farmanager.com/view.php?id=2764
}

Macro { description="^QX  Go to bottom of window";
  area="Editor"; key="CtrlQ-X";
  uid="92802A0A-E94F-4E4A-884B-A3378B6A1BEE";
  action=function() Keys"CtrlE" end;
}

Macro { description="^QR  Go to start of file";
  area="Editor"; key="CtrlQ-R";
  uid="2F3FFA03-D710-4201-9933-C37996ADAE0A";
  action=function() Keys"CtrlHome" end;
}

Macro { description="^QC  Go to end of file";
  area="Editor"; key="CtrlQ-C";
  uid="B981C223-E059-4396-A71B-A7B32F740196";
  action=function() Keys"CtrlEnd" end;
}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
local E = {}
local LastPosId
local SavedLastPos

for i=1,far.AdvControl(F.ACTL_GETWINDOWCOUNT) do
  local w = far.AdvControl(F.ACTL_GETWINDOWINFO,i)
  if w.Type==F.WTYPE_EDITOR then
    editor.SubscribeChangeEvent(w.Id,true)
    E[w.Id] = {}
  end
end

local GetPosFlag
Event { description="turbo: LastPosition handler";
  group="EditorEvent";
  uid="8A337F5E-D742-49CA-9106-B0767CF4F904";
  action=function(id,Event,param)
    if Event==F.EE_READ then
      editor.SubscribeChangeEvent(nil,true)
      E[id] = {FileName=editor.GetFileName()}
    elseif Event==F.EE_CHANGE and E[id] then
      LastPosId = id
      E[id].LastPos = {CurLine=param.StringNumber,FileName=E[id].FileName}
      if param.Type==F.ECTYPE_CHANGED then
        GetPosFlag = true
      end
    elseif Event==F.EE_REDRAW and GetPosFlag then
      local LastPos = editor.GetInfo(id)
      if E[id].LastPos.CurLine==LastPos.CurLine then
        E[id].LastPos = LastPos
      end
      GetPosFlag = false
    elseif Event==F.EE_CLOSE then
      if id==LastPosId then SavedLastPos = E[id].LastPos end
      E[id] = nil
    end
  end;
}

--requires FAR 3 build >=3854 (EF_OPENMODE_USEEXISTING)
local EF = {EF_NONMODAL=1,EF_IMMEDIATERETURN=1,EF_OPENMODE_USEEXISTING=1}
local function gotoLastPosition()
  if LastPosId then
    local Pos = E[LastPosId] and E[LastPosId].LastPos or SavedLastPos
    mf.postmacro(function()
      if far.AdvControl(F.ACTL_GETWINDOWINFO).Type==F.WTYPE_EDITOR
         and editor.GetInfo().EditorID==Pos.EditorID then
        editor.SetPosition (nil,Pos)
      end
    end)
    editor.Editor(Pos.FileName,nil,nil,nil,nil,nil,EF)
    --todo: ??fix modal editor
  end
end

Macro { description="^QP  Last position";
  area="Editor"; key="CtrlQ-P ShiftF5";
  uid="19343E14-97F7-47EA-9AEC-55168546FD1C";
  action=function()
    local info = editor.GetInfo()
    local e = E[info.EditorID]
    if e.LastPos then
      editor.SetPosition(nil,e.LastPos)
      e.LastPos = info
    else
      gotoLastPosition()
    end
  end;
}

Macro { description="^QP  Last position";
  area="Shell Info QView Tree"; key="CtrlQ-P";
  uid="BF24FAE0-A1B5-4987-8ACA-F1067743CB37";
  action=gotoLastPosition;
}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
Macro { description="^QY  Delete to end of line";
  area="Editor"; key="CtrlQ-Y";
  uid="8110CC74-9E67-4509-B4FF-612F4E1CEC8A";
  action=function() Keys"CtrlK" end;
}

Macro { description="^Q-DEL Delete to start of Line";
  area="Editor"; key="CtrlQ-Del";
  uid="71EF50A3-90C3-4510-BE26-652BF8BDBADF";
  action=function() Keys"CtrlShiftBS" end;
}

local function isVisible(PosData) --checks Pos visibility on current screen
  assert(PosData.CurLine)
  local info = editor.GetInfo()
  local top = info.TopScreenLine
  local bottom = top+info.WindowSizeY-1
  --local left = info.LeftPos
  --local right = left+info.WindowSizeX-1
  if PosData.CurLine>=top and PosData.CurLine<=bottom 
     --and PosData.CurPos>=left and PosData.CurPos<=right 
     then
    return true
  end
end

local function gotoBlockBorder(param)
  local sel = editor.GetSelection()
  if sel then
    local PosData = param=="B" and {CurPos=sel.StartPos, CurLine=sel.StartLine}
                 or param=="K" and {CurPos=sel.EndPos+1, CurLine=sel.EndLine}
    if not isVisible(PosData) then
      --todo ?? LeftPos
      local TopScreenLine = sel.StartLine-3
      PosData.TopScreenLine = TopScreenLine>0 and TopScreenLine or 1
    end  
    editor.SetPosition(nil,PosData)
  end
end

Macro { description="^QB  Go to start of block";
  area="Editor"; key="CtrlQ-B";
  uid="F3C8ED5C-8DC8-4B5B-B7A8-A20B153FF5BA";
  action=function() gotoBlockBorder"B" end;
}

Macro { description="^QK  Go to end of block";
  area="Editor"; key="CtrlQ-K";
  uid="9C991122-FC4B-47DB-9F14-61238979074F";
  action=function() gotoBlockBorder"K" end;
}

--^QT Tab mode on/off
--^QI Autoindent on/off

Macro { description="^QL  Restore line";
  area="Editor"; key="CtrlQ-L";
  uid="DB0DB4EA-E972-439D-8C46-50739DFBC99A";
  action=function() editor.UndoRedo(nil,F.EUR_UNDO) end; --todo
}

local AKey --http://bugs.farmanager.com/view.php?id=2309
Macro { description="^Q-n Find place marker";
  area="Editor"; key="CtrlQ-1 CtrlQ-2 CtrlQ-3 CtrlQ-4 CtrlQ-5 CtrlQ-6 CtrlQ-7 CtrlQ-8 CtrlQ-9 CtrlQ-0";
  uid="FA217A89-A4F4-4A1A-9E9D-26AC0B1E579B";
  condition=function(key) AKey = key; return true end;
  action=function() Keys(AKey:gsub("Q-","")) end;
}

--N="AltF8"; --^QN Go to line n

local function matchBracket(cmd)
  Plugin.Call("B66C436D-E657-419C-86D3-6834B9ACF3D2",cmd) --Brackets
end

Macro { description="^Q[  Pair matching right [Brackets plugin]";
  area="Editor"; key="CtrlQ-[";
  uid="74231CFF-4FEB-434D-9732-594D547A14DB";
  action=function() matchBracket"SearchFwd" end;
}

Macro { description="^Q]  Pair matching left [Brackets plugin]";
  area="Editor"; key="CtrlQ-]";
  uid="11A68D34-9487-421E-9520-DE6BABEC60FE";
  action=function() matchBracket"SearchBack" end;
}

Macro { description="^Q{";
  area="Editor"; key="CtrlQ-Shift[ CtrlQ-{";
  uid="A295957A-7322-4430-B61A-2CDF44AFC85B";
  action=function() matchBracket"SelectFwd" end;
}

Macro { description="^Q}";
  area="Editor"; key="CtrlQ-Shift] CtrlQ-}";
  uid="485BE381-008E-490F-8A06-4A89732BC79E";
  action=function() matchBracket"SelectBack" end;
}

Macro { description="^QF  Find";
  area="Editor"; key="CtrlQ-F";
  uid="92829369-CFDD-4494-A179-B3DC93C3349C";
  action=function() Keys"F7" end;
}

Macro { description="^QA  Find & replace";
  area="Editor"; key="CtrlQ-A";
  uid="D6928262-5296-4CBC-8127-C8959DEBDA04";
  action=function() Keys"CtrlF7" end;
}

local function posCurLineTo(param)
  local PosData = editor.GetInfo()
  PosData.TopScreenLine = PosData.CurLine - (param=="T" and 0 or "U" and PosData.WindowSizeY-1)
  editor.SetPosition(nil,PosData)
end

Macro { description="^QT  Pos current line to the top of the window";
  area="Editor"; key="CtrlQ-T";
  uid="03C21E0B-9AD9-4650-ADDF-CD2B6B1E8115";
  action=function() posCurLineTo"T" end;
}

Macro { description="^QU  Pos current line to the bottom of the window";
  area="Editor"; key="CtrlQ-U";
  uid="5EAE36F3-3462-42E5-AC00-425EEADF06AF";
  action=function() posCurLineTo"U" end;
}

--[[
Use ^QV to position the cursor back at the beginning
marker's location preVious to moving, copying, or
deleting a block. Then check to verify that the move
was what you intended, and use regular editing commands
as necessary to reformat the original area.

Press ^QV to return the cursor to your preVious
place in the text. This spot is not the same as your
cursor's original position except when only one
occurrence of your string is found. If you're going to
use ^L many times, mark your original position in
the text with a place marker so that you can return
there quickly.
--]]
